/*
 * mango64LED_DD.c
 */
#include <linux/init.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/gpio.h>
#include <plat/gpio-cfg.h>

MODULE_LICENSE("Dual BSD/GPL");

#define MANGO64_LED_MAJOR_NUMBER  240
#define MANGO64_LED_DEVICE_NAME   "mango64_led_device"

// int (*open) (struct inode *, struct file *);
int mango64_led_open(struct inode * inode_p, struct file * file_p)
{
    printk("[DEV] mango64_led_open() major:%d, minor:%d\n",
           MAJOR(inode_p->i_rdev), MINOR(inode_p->i_rdev));
	return 0;
}

// int (*release) (struct inode *, struct file *);
int mango64_led_release(struct inode * inode_p, struct file * file_p)
{
    printk("[DEV] mango64_led_release()\n");
	return 0;
}

static void led_1_on(void)
{
    gpio_set_value(S3C64XX_GPM(5), 0);
}

static void led_1_off(void)
{
    gpio_set_value(S3C64XX_GPM(5), 1);
}

static void led_2_on(void)
{
    gpio_set_value(S3C64XX_GPL(8), 0);
}

static void led_2_off(void)
{
    gpio_set_value(S3C64XX_GPL(8), 1);
}

// int (*ioctl) (struct inode *, struct file *, unsigned int, unsigned long);
int mango64_led_ioctl(struct inode * inode_p, struct file * file_p,
                      unsigned int command, unsigned long argument)
{
    printk("[DEV] mango64_led_ioctl() command:%x, argument:%x\n", command, (int)argument);

    switch(command)
    {
    case 1:
        if(0 == argument)
        {
            printk("[DEV] IND LED 1 Control, Off\n");
            led_1_off();
        }
        else if(1 == argument)
        {
            printk("[DEV] IND LED 1 Control, On\n");
            led_1_on();
        }
        else
        {
            printk("[DEV] Invalid IOCTL Argument: %x\n", (int)argument);
            return -1;
        }
        break;
    case 2:
        if(0 == argument)
        {
            printk("[DEV] IND LED 2 Control, Off\n");
            led_2_off();
        }
        else if(1 == argument)
        {
            printk("[DEV] IND LED 2 Control, On\n");
            led_2_on();
        }
        else
        {
            printk("[DEV] Invalid IOCTL Argument: %x\n", (int)argument);
            return -1;
        }
        break;
    default:
        printk("[DEV] Invalid IOCTL Command: %x\n", command);
        return -1;
        break;
    }

    return 0;
}

struct file_operations mango64_led_file_oper =
{
    .owner   = THIS_MODULE,
    .open    = mango64_led_open,
    .ioctl   = mango64_led_ioctl,
    .release = mango64_led_release,
};

int mango64_led_init(void)
{
    int retVal;
    printk("[DEV] mango64_led_init() is called.\n");

    gpio_direction_output(S3C64XX_GPM(5), S3C_GPIO_OUTPUT);
    gpio_direction_output(S3C64XX_GPL(8), S3C_GPIO_OUTPUT);

    led_1_on();
    led_2_on();

    retVal = register_chrdev(MANGO64_LED_MAJOR_NUMBER, MANGO64_LED_DEVICE_NAME, &mango64_led_file_oper);

    printk("[DEV] return value: %d\n", retVal);
    
    return retVal;
}

void mango64_led_exit(void)
{
    printk("[DEV] mango64_led_exit() is called.\n");

	unregister_chrdev(MANGO64_LED_MAJOR_NUMBER, MANGO64_LED_DEVICE_NAME);
}

module_init(mango64_led_init);
module_exit(mango64_led_exit);

